// Knihovna pro prci s I2C displejem Philips SAA1064
// displej v multiplexovanm reimu (4 slice)
// Prce s I2C EEPROM (small/big)
// Prce s MCP23016
//
// (F) Freeware, Tranzistor electronic 2004
// --------------------------------------------------

#include <pic18.h>
#include "I2CPeriph.h"
#include "Delay.h"


#ifdef SAA1064Dis

const Num7[]={Seg7_0,Seg7_1,Seg7_2,Seg7_3,Seg7_4,Seg7_5,
              Seg7_6,Seg7_7,Seg7_8,Seg7_9,Seg7_A,Seg7_b,
              Seg7_C,Seg7_d,Seg7_E,Seg7_F};

#endif

static bit SCL  @ (unsigned)&PORTC*8+3;		// Definice vstup I2C
static bit SDA  @ (unsigned)&PORTC*8+4;
static bit TSCL @ (unsigned)&TRISC*8+3;
static bit TSDA @ (unsigned)&TRISC*8+4;

bit I2CStarted,I2CError;

unsigned char DisplayData[4];
unsigned char OldMCP23016KeyVal;

// ek na dokonen kolu I2C

void I2CWait(void)
{
  while(!SSPIF);
  SSPIF=0;
}

// Inicializace I2C

void I2CSetup(void)
{
  char i;
  SSPCON1=0x38;		// I2C master mode
  I2CStarted=0;		// Zatm se nic nedje
  I2CError=0;
  SSPSTAT=0x80;		// 100 kHz sbrnice (no slew rate control)
  SSPCON2=0;		// Vynulovat informan bity
  SSPADD=CrystalFreq/(400000);            // 100 kHz sbrnice, SSPADD = fosc/(4.100kHz)-1
  if(SDA) return;		// Pokud bez chyby, tak konec
  PORTC&=0xE7;		// Nastaven SDA a SCL na nulu
  SSPEN=0;			// Zkaz I2C
  for(i=9;i!=0;i--)	// Maximln 9 impuls,pak u chyba nejde opravit
  {
    TSCL=0;			// Hodiny dol
    DelayUs(40);		// Chvilku ekej (40us)
    TSCL=1;			// Hodiny nahodu
    DelayUs(40);		// Chvilku ekej (40us)
    if(SDA) break;
  }
  SSPEN=1;			// Povol I2C
}

// Provede podmnku START na I2C

void I2CStart(void)
{
  if(I2CStarted) RSEN=1;// Generovn podmnky REPEATED START
  else			// Pokud je prvn
  {
    if((!SCL)|(!SDA))	// Pokud je sbrnice staen
    { 
      I2CError=1;		// Tak dolo k chyb
      return;		// a konec
    }				// Jinak, pokud je ve OK
    SEN=1;			// Generovn podmnky START
  }
  I2CStarted=1;		// Nastartovno
  I2CWait();		// ekej dokud nen dokoneno
//  DelayUs(20);
}

// Provede podmnku STOP na I2C

void I2CStop(void)
{
  PEN=1;			// Generovn podmnky STOP
  I2CStarted=0;		// Zastaveno
  I2CWait();		// ekej dokud nen dokoneno
//  DelayUs(20);
}

// Vyle byte na I2C

void I2COut(char b)
{
  SSPBUF=b;
  I2CWait();
//  DelayUs(20);
}

// Pete byte z I2C

char I2CIn(void)
{
  char b;
  RCEN=1;					// Povol pjem, start pjmu
  I2CWait();				// ekej na byte
//  DelayUs(20);
  b=SSPBUF;					// Peti ho z bufferu
  ACKEN=1;					// Acknowledge
  I2CWait();				// ekej dokud nen dokoneno
//  DelayUs(20);
  return b;
}

bit SetI2CEEPROMAdr(unsigned int EEAdr)
{
  unsigned char LoAdr,HiAdr;

  LoAdr=EEAdr;
  HiAdr=EEAdr>>8;
  I2CStart();
  #if(I2CEEPROMType==SmallEEPROM)
    HiAdr<<=1;
    I2COut(HiAdr|0xA0);
  #else
    I2COut(0xA0);
    I2COut(HiAdr);
  #endif
  if(ACKSTAT) { I2CStop();return 0; };
  I2COut(LoAdr);
  return 1;
}

bit SetI2CEEPROMRd(unsigned int EEAdr)
{
  unsigned char LoAdr,HiAdr;

  LoAdr=EEAdr;
  HiAdr=EEAdr>>8;
  I2CStart();
  #if(I2CEEPROMType==SmallEEPROM)
    HiAdr<<=1;
    I2COut(HiAdr|0xA0);
  #else
    I2COut(0xA0);
    I2COut(HiAdr);
  #endif
  if(ACKSTAT) { I2CStop();return 0; };
  I2COut(LoAdr);
  I2CStart();
  ACKDT=0;
  #if(I2CEEPROMType==SmallEEPROM)
    I2COut(HiAdr|0xA1);
  #else
    I2COut(0xA1);
  #endif
  return 1;
}

char DeviceExists(unsigned char DevAdr)
{
//  I2CSetup();		// Nastaven I2C
  I2CStart();		// Podmnka START, potek komunikace
  if(I2CError)		// Pokud je sbrnice obsazen
  {
    I2CStop();
    return I2C_sError;	// Chyba na zazen
  }
  I2COut(DevAdr);		// Adresa EEPROM, dummy write
  if(ACKSTAT)		// Pokud nen neodpov => Odpojeno
  {
    I2CStop();
    return I2C_sNoAck;	// Opojeno (dn zazen nen pipojeno na konektoru)
  }
  I2CStop();
  return I2C_sOK;		// Ve v podku
}

#ifdef SAA1064Dis

void SetDisplay(unsigned char Control)
{
  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(SAA1064);	// SAA1064 adresa
  I2COut(0x00);			// Control register
  I2COut(Control);
  I2CStop();
}

void WriteDisplay(void)
{
  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(SAA1064);	// SAA1064 adresa
  I2COut(0x01);			// Pozice prvnho sla
  I2COut(DisplayData[3]);	// Data
  I2COut(DisplayData[2]);
  I2COut(DisplayData[1]);
  I2COut(DisplayData[0]);
  I2CStop();
}

void ClearDisplay(void)
{
  DisplayData[3]=Seg7_nic;
  DisplayData[2]=Seg7_nic;
  DisplayData[1]=Seg7_nic;
  DisplayData[0]=Seg7_nic;
  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(SAA1064);	// SAA1046 adresa
  I2COut(0x01);			// Pozice prvnho sla
  I2COut(DisplayData[3]);	// Data
  I2COut(DisplayData[2]);
  I2COut(DisplayData[1]);
  I2COut(DisplayData[0]);
  I2CStop();
}

char CharToSeg7(char ch)
{
  switch(ch)
  {
    case '0':return Seg7_0;
    case '1':return Seg7_1;
    case '2':return Seg7_2;
    case '3':return Seg7_3;
    case '4':return Seg7_4;
    case '5':return Seg7_5;
    case '6':return Seg7_6;
    case '7':return Seg7_7;
    case '8':return Seg7_8;
    case '9':return Seg7_9;
    case 'A':return Seg7_A;
    case 'b':return Seg7_b;
    case 'c':return Seg7_c;
    case 'C':return Seg7_C;
    case 'd':return Seg7_d;
    case 'E':return Seg7_E;
    case 'F':return Seg7_F;
    case 'g':return Seg7_g;
    case 'h':return Seg7_h;
    case 'H':return Seg7_H;
    case 'i':return Seg7_i;
    case 'J':return Seg7_J;
    case 'L':return Seg7_L;
    case 'M':return Seg7_M;
    case 'n':return Seg7_n;
    case 'o':return Seg7_o;
    case 'O':return Seg7_0;
    case 'P':return Seg7_P;
    case 'q':return Seg7_q;
    case 'r':return Seg7_r;
    case 'S':return Seg7_S;
    case 't':return Seg7_t;
    case 'u':return Seg7_u;
    case 'U':return Seg7_U;
    case 'Y':return Seg7_Y;
    case 'Z':return Seg7_Z;

    case '=':return Seg7_mriz;
    case '-':return Seg7_minus;
    case ' ':return Seg7_space;
    case '_':return Seg7_podtrzitko;
    case '.':return Seg7_tecka;
    case '':return Seg7_stupnu;
  }
  return Seg7_space;
}

void putsDisplay(const char * s)
{
  unsigned char i=4;
  while(i--) DisplayData[i]=CharToSeg7(*(s+i));
  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(SAA1064);            // SAA1064 adresa
  I2COut(0x01);               // Pozice prvnho sla
  I2COut(DisplayData[3]);     // Data
  I2COut(DisplayData[2]);
  I2COut(DisplayData[1]);
  I2COut(DisplayData[0]);
  I2CStop();
}

#endif

#ifdef MCPKeyboard

void SetupMCP23016Key64()
{
  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(MCP23016);		// MCP23016 adresa
  I2COut(MCP23016_OLAT0);	// Bude zpis na LAT0
  I2COut(0);

  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(MCP23016);		// MCP23016 adresa
  I2COut(MCP23016_IODIR0);	// Smr registr
  I2COut(0);			// Registr 0 vstupn
  I2COut(0xFF);			// Registr 1 vstupn

  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(MCP23016);		// MCP23016 adresa
  I2COut(MCP23016_IPOL1);	// Polarita portu 1
  I2COut(0xFF);			// invertovan

  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(MCP23016);		// MCP23016 adresa
  I2COut(MCP23016_IOCON0);	// Rychlost detekce zmny portu - peruen
  I2COut(0);			// Pomal
  I2COut(0);			// Pomal
  I2CStop();
}

unsigned char ReadMCP23016Keyboard64(unsigned char Sloupec)
{
  unsigned char Radek;
  I2CStart();
  if((I2CError)||(!I2CStarted)) return 0xFF;

  I2COut(MCP23016);		// MCP23016 adresa
  I2COut(MCP23016_OLAT0);	// Zpis na port 0
  I2COut(Sloupec);
  I2CStop();

  DelayMs(1);

  I2CStart();
  if((I2CError)||(!I2CStarted)) return 0xFF;

  I2COut(MCP23016);       // MCP23016 adresa
  I2COut(MCP23016_GP0);   // Bude se st port 0 a 1
  I2CStop();              // MCP23016 vyaduje STOP!!! (restart nesta)

  I2CStart();
  if((I2CError)||(!I2CStarted)) return 0xFF;

  I2COut(MCP23016+1);		// MCP23016 adresa
  DelayUs(100);
  ACKDT=0;
  I2CIn();				// Vyistit od peruen
  DelayUs(100);
  ACKDT=1;
  Radek=I2CIn();			// Pest a vyistit peruen
  I2CStop();
  return Radek;
}

unsigned char GetKeyMCP23016Key64()
{
  unsigned char i,j,k,n;
  i=8;
  while(i--)
  {
    j=1<<i;
    j=ReadMCP23016Keyboard64(~j);
    if(j)
    {
      k=i*8;
      n=0x01;
      while(n)
      {
        if((j&n)&&(OldMCP23016KeyVal!=k)) return OldMCP23016KeyVal=k;
        k++;
        n<<=1;
      }
    }
  }
  return 0xFF;
}

void SetupMCP23016Key60()
{
  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(MCP23016);           // MCP23016 adresa
  I2COut(MCP23016_OLAT0);     // Bude zpis na LAT0
  I2COut(0);

  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(MCP23016);           // MCP23016 adresa
  I2COut(MCP23016_IODIR0);    // Smr registr
  I2COut(0b11000000);         // Registr 0 vstupn, a na 2 MSB vstupn
  I2COut(0xFF);               // Registr 1 vstupn

  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(MCP23016);           // MCP23016 adresa
  I2COut(MCP23016_IPOL0);     // Polarita portu 0 a 1
  I2COut(0b11000000);         // invertovan
  I2COut(0xFF);

  I2CStart();
  if((I2CError)||(!I2CStarted)) return;

  I2COut(MCP23016);           // MCP23016 adresa
  I2COut(MCP23016_IOCON0);    // Rychlost detekce zmny portu - peruen
  I2COut(0);                  // Pomal
  I2COut(0);                  // Pomal
  I2CStop();
}

unsigned int ReadMCP23016Keyboard60(unsigned char Sloupec)
{
  unsigned int Radek;
  I2CStart();
  if((I2CError)||(!I2CStarted)) return 0xFFFF;

  I2COut(MCP23016);           // MCP23016 adresa
  I2COut(MCP23016_OLAT0);     // Zpis na port 0
  I2COut(Sloupec);
  I2CStop();

  DelayMs(1);

  I2CStart();
  if((I2CError)||(!I2CStarted)) return 0xFFFF;

  I2COut(MCP23016);           // MCP23016 adresa
  I2COut(MCP23016_GP0);       // Bude se st port 0 a 1
  I2CStop();                  // MCP23016 vyaduje STOP!!! (restart nesta)

  I2CStart();
  if((I2CError)||(!I2CStarted)) return 0xFFFF;

  I2COut(MCP23016+1);         // MCP23016 adresa
  DelayUs(100);
  ACKDT=0;
  Radek=I2CIn();              // Pest vrchn st a vyistit od peruen
  Radek&=0xC0;
  Radek<<=2;
  DelayUs(100);
  ACKDT=1;
  Radek|=I2CIn();             // Pest doln st a vyistit peruen
  I2CStop();
  return Radek;
}

unsigned char GetKeyMCP23016Key60()
{
  unsigned char i,k;
  unsigned int j,n;
  i=0;
  while(i<10)
  {
    k=1<<i;
    j=ReadMCP23016Keyboard60(~k);
    if(j)
    {
      k=i*10;
      n=0x01;
      while(n&0x3FF)
      {
        if((j&n)/*&&(OldMCP23016KeyVal!=k)*/) return OldMCP23016KeyVal=k+1;
        k++;
        n<<=1;
      }
    }
    i++;
  }
  return 0;
}

#endif

#ifdef UseTDA8444

void SetVoltTDA8444(unsigned char Vystup, unsigned char Value)
{
  I2CStart();
  I2COut(TDA8444);
  I2COut(Vystup&7);
  I2COut(Value>>2);
  I2CStop();
}

void SetRangeTDA8444(unsigned char Start, unsigned char Stop, unsigned char * Values)
{
  I2CStart();
  I2COut(TDA8444);
  I2COut(Start);
  while(Start<=Stop) { I2COut((*(Values++))>>2);Values++;Start++; };
  I2CStop();
}

#endif
