#ifndef _LCD_H_
#define _LCD_H_

/*
 *	LCD interface header file
 */

/* 	Defining CHECKBUSY will check if the LCD is busy. The RW bit of the 
 * 	LCD must connected to a port of the processor for the check busy
 * 	process to work.
 * 
 * 	If CHECKBUSY is not defined it will instead use a delay loop.
 * 	The RW bit of the LCD does not need to connected in this case.
 */

#define CHECKBUSY	    1

#ifdef CHECKBUSY
	#define	LCD_WAIT lcd_check_busy()
#else
	#define LCD_WAIT DelayMs(5)
#endif

#define MESSAGE_LINE	  0x0

#define FOURBIT_MODE	  0x0
#define EIGHTBIT_MODE	  0x1
#define OUTPUT_PIN      0x0	
#define INPUT_PIN       0x1	
#define OUTPUT_DATA     0xF0	
#define INPUT_DATA      0xFF

#define LCDOneEN      0
#define LCDTwoEN      1
#define DisplayLen    16
#define LineLength	  8
#define DisplayType   LCDOneEN

#define LCD_RS        RA5
#define LCD_EN        RA2
#define LCD_RW        RA3 
#define LCD_DATA      PORTB
#define LCD_DATA_PORT PORTB 
#define LCD_RS_TRIS   TRISA5
#define LCD_EN_TRIS   TRISA2
#define LCD_RW_TRIS   TRISA3 
#define LCD_DATA_TRIS TRISB

#if(DisplayLen==16)
  #define LcdLn1		0x00
  #define LcdLn2		0x40
  #define LcdLn3		0x10
  #define LcdLn4		0x50
#endif
#if(DisplayLen==20)
  #define LcdLn1		0x00
  #define LcdLn2		0x40
  #define LcdLn3		0x14
  #define LcdLn4		0x54
#endif
#if(DisplayLen==40)
  #define LcdLn1		0x00
  #define LcdLn2		0x40
  #define LcdLn3		0x80
  #define LcdLn4		0xC0
#endif

#if(DisplayType==LCDOneEN)
  #define LCD_STROBE()	LCD_EN = 1; \
	  			DelayUs(5); \
		  		LCD_EN = 0

  #define LCD_STROBE_READ(value)	LCD_EN = 1; \
	  			DelayUs(5); \
		  		value=LCD_DATA_PORT; \
			  	LCD_EN = 0
#endif

#if(DisplayType==LCDTwoEN)
  #define LCD_STROBE()	if(ENSelect&1) LCD_EN1 = 1; \
          if(ENSelect&2) LCD_EN2 = 1; \
	  			DelayUs(15); \
		  		LCD_EN1 = 0; \
          LCD_EN2 = 0

  #define LCD_STROBE_READ(value)	if(ENSelect&1) LCD_EN1 = 1; \
          if(ENSelect&2) LCD_EN2 = 1; \
	  			DelayUs(15); \
		  		value=LCD_DATA_PORT; \
			  	LCD_EN1 = 0; \
          LCD_EN2 = 0 
#endif

#define lcd_cursor(x)			    lcd_cmd(((x)&0x7F)|0x80)
#define lcd_clear()			      lcd_cmd(0x1)
#define lcd_putch(x)			    lcd_data(x)
#define lcd_goto(x)			      lcd_cmd(0x80+(x))
#define lcd_cursor_right()		lcd_cmd(0x14)
#define lcd_cursor_left()		  lcd_cmd(0x10)
#define lcd_display_shift()		lcd_cmd(0x1C)
#define lcd_home()            lcd_cmd(0x2)

extern unsigned char ENSelect;

extern void lcd_cmd(unsigned char);
extern void lcd_data(unsigned char);
extern void lcd_puts(const char * s);
extern void lcd_putsLn(const char * s);
extern void lcd_init(unsigned char);
extern void lcd_gotoLn(unsigned char pos);
extern void lcd_gotoXY(unsigned char Radek,unsigned char Sloupec);
extern void lcd_putnum(unsigned char num);
extern void lcd_puthex(unsigned char num);
extern void lcd_putuint(unsigned int num);
extern unsigned char lcd_read_cmd_nowait(void);
extern void lcd_LoadCGRAM(unsigned char Pozice,const unsigned char * Znak);

#endif

