// Vario a vkomr, zznam do EEPROM, komunikace s PC pes USB
// PIC18F242 @ 3.6864MHz

// Senzor
// p=(Vout+0.475)*22.22
//
// Digitln vstup A/D pevodnku
// p=(Vdig+0.475*Deli*RozsahDigAD/RozsahVstupAD)*22.22/Dli*RozsahVstupAD/RozsahDigAD
// p=(Vdig+0.475*27/66*2^18/2.048)*22.22*66/27*2.048/2^18
// p=(Vdig+24872.7273)*4.2438271605E-4

// Pevod vky na napt
// pH=ph.exp[-H-h/7998]
// H - Vka absolutn
// h - Vka ve kter se m referenn tlak, hladina moe h=0
// p - tlak, pH v na vce, ph v referenn vce
// H=7998.ln(101325/pH) absolutn vka v zvislosti na absolutnm tlaku v=0

// Formt EEPROM
// sla znamenajc absolutn vku
// signed integer 0-65536, vka + 10000 uloit do EEPROM
// Klov sla
// FFFF - Konec EEPROM (Konec zpisu)
// FFFE - Hlavika
// 0 (2) - Adresa konce (uint), nastavuje se pi zatku dalho zznamu, default FFFF
// 2 (4) - Tlak0 (float)
// 6 (2) - Vka startovaky (uint) + 10000
// 7 (1) - Ukldn
// 8 (1) - Reserved (free)

#include <pic18.h>
#include <math.h>
#include "lcd.h"
#include "delay.h"
#include "I2CPeriph.h"

__CONFIG(1,XT);
__CONFIG(2,PWRTDIS&BORV42&WDTDIS&WDTPS128);
__CONFIG(3,CCP2RC1);
__CONFIG(4,STVREN&DEBUGDIS&LVPDIS);
__CONFIG(5,UNPROTECT);
__CONFIG(6,WRTEN);
__CONFIG(7,TRU); 

#define KB_Nic    0
#define KB_Left   0x80
#define KB_Right  0x40
#define KB_Up     0x20
#define KB_Down   0x10
#define KB_Enter  0xC0
#define KB_Cancel 0x30

#define Piezo     RC2 // CCP1

#define Cas10ms     0xDC    // Pro krystal 3.6864 MHz, 65536-fosc/4000*10

#define CasZakmitu  5       // 50ms

#define EEPROM_Posun        0
#define EEPROM_Rozsah       EEPROM_Posun+4
#define EEPROM_Tlak0        EEPROM_Rozsah+4
#define EEPROM_VyskaKopce   EEPROM_Tlak0+4
#define EEPROM_MaxVyska     EEPROM_VyskaKopce+2
#define EEPROM_Zvuk         EEPROM_MaxVyska+2
#define EEPROM_MinPokles    EEPROM_Zvuk+1
#define EEPROM_MinVzestup   EEPROM_MinPokles+1
#define EEPROM_EESize       EEPROM_MinVzestup+1
#define EEPROM_Ukladani     EEPROM_EESize+1

__EEPROM_DATA(0xB0,0xDB,0xE9,0xC5,    // Posun=-7483.461
              0x73,0x08,0x83,0x3F);   // Rozsah=1.023695
__EEPROM_DATA(0x80,0xE6,0xC5,0x47,    // Tlak pepoten na hladinu moe, 101325pa
              0x84,0x03,              // Posledn zadan vka kopce 900m
              0x54,0x0B);             // Maximln letov hladina 2900m
__EEPROM_DATA(0x02,                   // Zvuk
              0,                      // Minimlnn pokles aby byl zvuk
              0,                      // Minimln vzestup aby byl zvuk
              4,                      // Velikost EEPROM 0-24C32,1-64,2-128,3-256,4-512
              2,                      // Ukldn do EEPROM, kad n-t men
              0,0,0);

#define cmdStop         1
#define cmdRead         2
#define cmdData         3
#define cmdGetParam     4
#define cmdParam        5

#define MenuHV          0
#define MenuVario       1
#define MenuVyska       2
#define MenuTlak        3
#define MenuHmax        4
#define MenuNejvyse     5
#define MenuBeep        6
#define MenuPokles      7
#define MenuVzestup     8
#define MenuPosun       9
#define MenuRozsah     10
#define MenuBaterka    11
#define MenuCas        12
#define MenuUkladani   13
#define MenuPamet      14

#define MenuEnd        14

typedef union
{
  unsigned char Byte[4];
  double Real;
} TDouble4;

typedef union
{
  unsigned char Byte[2];
  unsigned int Int;
} TByteInt;

typedef union
{
  unsigned char Byte[4];
  signed long Long;
} TByteSLong;

unsigned near char Cas,CasZvuku,CasK,CasMereni;
unsigned near char OldKey,Menu,PocetVarovani,Zvuk,PocetZv,MinPokles,MinVzestup;
unsigned near char Ukladani,CasUkladani;
unsigned near int CasLetu=0;

unsigned char EDData[9];
signed char VarHistory[8];
unsigned int AdresaEE,PosledniEE,MaxMem;

near bit NeedUpdate,TempBit,UpdateLCD;

double near Tlak,Vyska,StaraVyska;
TDouble4 near Posun,Rozsah,Tlak0;
TByteInt near VyskaKopce,MaxVyska;
float VyskaDif;
double NejmensiTlak,TlakNaStartu;

unsigned char OutCommand,InPosition,InputBuffer[20];
unsigned int OutPosition;

void Sound(unsigned char Period,unsigned char Delka)
{
  PR2=Period;
  CasZvuku=Delka;
  CCPR1L=Period>>1;
  T2CON=0b00000111;     // 3686400 Hz/(4.4.4)=> fmax=57600; fmin=225Hz
  CCP1CON=0b00001111;
}

void NoSound()
{
  CCP1CON=0;Piezo=0;
}

void interrupt Preruseni()
{
  unsigned char i;
  if(TMR1IF)
  {
    TMR1H+=Cas10ms;      // Kach 10ms se zvt as o jedniku
    TMR1IF=0;
    if(CasK) CasK--;
    if(CasZvuku)
    {
      if(!(--CasZvuku)) NoSound();
    }
    CasMereni++;
    if(CasMereni>50) { NeedUpdate=1;CasMereni=0;CasLetu++; }
    if(PocetZv)
    {
      if(!((--PocetZv)&7))
      {
        CasZvuku=5;
        CCP1CON=0b00001111;
      }
    }
  }
  if(RBIF)
  {
    i=PORTB;
    RBIF=0;
    CasK=CasZakmitu;
  }
  ADIF=0;
  if(RCIF)
  {
    InputBuffer[InPosition]=RCREG;RCIF=0;
    InPosition++;
    switch(InputBuffer[0])
    {
      case cmdStop:{ InPosition=0;OutCommand=0;OutPosition=0;break; }
      case cmdRead:{ InPosition=0;OutCommand=cmdData;OutPosition=0;break; }
      case cmdGetParam:{ InPosition=0;OutCommand=cmdParam;OutPosition=0;break; }
      default:InPosition=0;
    }
  }
}

unsigned char ReadKey()
{
  if(CasK) return OldKey;
  OldKey=(~PORTB)&0xF0;
  return OldKey;
}

bit EditNumber(unsigned char Start,unsigned char Len)
{
  unsigned char i,Key,Pozice;

  lcd_cmd(0x0F);
  Pozice=0; 
  lcd_goto(Start);
  i=0;
  TempBit=0;
  while(i<Len)
  { 
    if(EDData[i]) TempBit=1;
    if(i==Len-1) TempBit=1;
    if(TempBit) lcd_putch(EDData[i]+'0');
           else lcd_putch(' ');
    i++;
  }
  lcd_goto(Pozice+Start);

  Key=ReadKey();
  while(1)
  {
    i=ReadKey();
    if(i!=Key)
    {
      Key=i;
      if(i) Sound(50,5);
      if(Key==KB_Left) if(Pozice) Pozice--;
      if(Key==KB_Right) if(Pozice<Len-1) Pozice++;
      if(Key==KB_Up)
      {
        EDData[Pozice]++;
        if(EDData[Pozice]>9) EDData[Pozice]=0;
      }
      if(Key==KB_Down)
      {
        EDData[Pozice]--;
        if(EDData[Pozice]>9) EDData[Pozice]=9;
      }
      if(Key==KB_Enter) { lcd_cmd(0x0C);return 1;}
      if(Key==KB_Cancel) { lcd_cmd(0x0C);return 0;}
      lcd_goto(Start);
      i=0;
      TempBit=0;
      while(i<Len)
      { 
        if(EDData[i]) TempBit=1;
        if(i==Len-1) TempBit=1;
        if(TempBit) lcd_putch(EDData[i]+'0');
               else lcd_putch(' ');
        i++;
      }
      lcd_goto(Pozice+Start);
    }
  }
  return 0;
}

void PrintDouble(double Cislo,unsigned char DM)
{
  unsigned long Cele;
  unsigned char i;
  unsigned char Zn;
  i=DM;
  while(i--) Cislo*=10;
  Zn=0;
  if(Cislo<0) { Cislo=-Cislo;Zn=1; }
  Cele=(unsigned long)Cislo;
  if(Cele>9999999) Cele=9999999;
  i=7;
  while(i--)
  {
    EDData[i]=Cele%10;
    Cele/=10;
  }
  lcd_goto(LcdLn2);
  TempBit=0;
  i=7;
  while(i--)
  {
    if((EDData[6-i])||(i==DM)) TempBit=1;
    if(TempBit) lcd_putch(EDData[6-i]+'0');
           else if((Zn)&&(EDData[7-i])) lcd_putch('-');
                                   else lcd_putch(' ');
    if((i==DM)&&(i)) lcd_putch('.');
  }
}

void ZmerBaterku()
{
  unsigned char i;
  ADCON1=0b00001110;  // AN0 m baterku, ostatn jsou digitln
  ADCON0=0b11000001;  // Zapnout A/D
  DelayMs(10);
  lcd_goto(LcdLn1);
  lcd_puts("Baterka");
  i=8;
  lcd_goto(LcdLn2);
  while(i--) lcd_putch(' ');
  GODONE=1;
  ADIF=0;
  ADIE=1;
  SLEEP();
  ADIE=0;
  i=(ADRESH-128)/11;
  if(i>8) i=8;
  ADON=0;
  lcd_goto(LcdLn2);
  while(i--) lcd_putch(0xFF);
}

void ZobrazVar()
{
  unsigned int i;
  if(VyskaDif>=0)
  {
    i=(unsigned int)(VyskaDif*10);
    if(i>999) i=999;
  }
  else 
  { 
    i=(unsigned int)(-VyskaDif*10);
    if(i>99) i=99;
    lcd_putch('-');
  }
  if(i>99)
  {
    lcd_putch(i/100+'0');
    i%=100;
  }
  lcd_putch(i/10+'0');
  lcd_putch('.');
  lcd_putch(i%10+'0');
}

void PrintSint(signed int Cislo)
{
  unsigned int Num;
  //unsigned char i;
  if(Cislo<0)
  {
    Cislo=-Cislo;
    lcd_putch('-');
    Num=Cislo;
    if(Num>9999) Num=9999;
  }
  else Num=Cislo;
  lcd_putuint(Num);
  /*TempBit=0;
  i=4;
  while(i--)
  {
    if((Num>999)||(TempBit)||(!i)) 
    {
      lcd_putch(Num/1000+'0');
      TempBit=1;
    }
    else lcd_putch(' ');
    Num%=1000;
    Num*=10;
  }*/
}

void ZobrazHV()
{
  signed int VyskaRel;
  lcd_puts("Hr=");
  VyskaRel=(signed int)(7998.0*log(Tlak0.Real/Tlak)+0.5)-VyskaKopce.Int;
  PrintSint(VyskaRel);
  lcd_goto(LcdLn2);
  lcd_puts("Var=");
  ZobrazVar();
}

void ZobrazVario()
{
  unsigned char i,j;
  lcd_puts("Var=");
  ZobrazVar();
  lcd_goto(LcdLn2);
  i=8;
  while(i--) lcd_putch(i);

  j=8;
  while(j--)
  {
    lcd_cmd(0x40+j*8);
    i=8;
    while(i--) 
      if(VarHistory[j]>i) lcd_data(0xFF);
                     else lcd_data(0);
  }
}

void ZobrazVysku()
{
  lcd_puts("Vyska nm");
  PrintDouble(Vyska,1);
}

void ZobrazTlak()
{
  lcd_puts("Tlak hpa");
  lcd_goto(LcdLn2);
  PrintDouble(Tlak/100.0,1);
}

void ZobrazHmax()
{
  lcd_puts("H max");
  lcd_goto(LcdLn2);
  lcd_putuint(MaxVyska.Int);
}

void ZobrazNejvyse()
{
  lcd_puts("Nejvyse");
  PrintDouble(7998.0*log(Tlak0.Real/NejmensiTlak)-(double)VyskaKopce.Int,1);
}

void ZobrazPamet()
{
  unsigned long Cislo;
  lcd_puts("Pamet");
  lcd_goto(LcdLn2);
  if(AdresaEE)
  {
    lcd_puts("Rec:");
    Cislo=AdresaEE+2;
  }
  else
  {
    lcd_puts("Obs:");
    Cislo=PosledniEE+2;
    if(PosledniEE==0xFFFF) { lcd_putch('?');return; }
  }
  lcd_putnum(Cislo*100/MaxMem);
  lcd_putch('%');
}

void SelectBeep()
{
  lcd_puts("Zvuk");
  lcd_goto(LcdLn2);
  switch(Zvuk)
  {
    case 0:lcd_puts("Vypnut");break;
    case 1:lcd_puts("f+Doba");break;
    case 2:lcd_puts("f+Pocet");break;
    case 3:lcd_puts("Doba");break;
    case 4:lcd_puts("Pocet");break;
  }
}

void PoklesBeep()
{
  lcd_puts("Min.Pokl");
  lcd_goto(LcdLn2);
  lcd_putnum(MinPokles);
}

void VzestupBeep()
{
  lcd_puts("Min.Vzes");
  lcd_goto(LcdLn2);
  lcd_putnum(MinVzestup);
}

void NastUkladani()
{
  lcd_puts("Ukladani");
  lcd_goto(LcdLn2);
  lcd_putnum(Ukladani);
}

void ZobrazPosun()
{
  lcd_puts("Kalibr.");
  lcd_goto(LcdLn2);
  lcd_puts("Posun:");
}

void ZobrazRozsah()
{
  lcd_puts("Kalibr.");
  lcd_goto(LcdLn2);
  lcd_puts("Rozsah:");
}

void Long2Text(unsigned long Cislo)
{
  EDData[0]=(Cislo/100000)%10;
  EDData[1]=(Cislo/10000)%10;
  EDData[2]=(Cislo/1000)%10;
  EDData[3]=(Cislo/100)%10;
  EDData[4]=(Cislo/10)%10;
  EDData[5]=Cislo%10;
}

unsigned long Text2Long(unsigned char Pocet)
{
  unsigned long Vysl=0;
  unsigned char i=0;
  while(i<Pocet)
  {
    Vysl*=10;
    Vysl+=EDData[i];
    i++;
  }
  return Vysl;
}

void Byte2Text(unsigned char Cislo)
{
  EDData[0]=Cislo/100;
  EDData[1]=(Cislo/10)%10;
  EDData[2]=Cislo%10;
}

void VarujVyskou()
{
  unsigned char i;
  lcd_clear();
  lcd_puts("Vyska!!!");
  i=40;
  while(i--)
  {
    Sound(i+10,9);
    DelayMs(10);
  }
  i=40;
  while(i--)
  {
    Sound(50-i,9);
    DelayMs(10);
  }
  PocetVarovani--;
}

void Beep1()
{
  signed char VyskaTonu;
  VyskaTonu=(signed char)VyskaDif;
  if(VyskaDif>20.0) VyskaTonu=20;
  if(VyskaDif<-20.0) VyskaTonu=-20;
  if(VyskaDif>0) if(VyskaTonu<10) Sound(50-VyskaTonu*2,VyskaTonu*2+5);
                             else Sound(40-VyskaTonu,VyskaTonu*2+5);
  if(VyskaDif<0) Sound(120-VyskaTonu*6,-VyskaTonu*2+5);
}

void Beep2()
{
  signed char VyskaTonu;
  VyskaTonu=(signed char)VyskaDif;
  if(VyskaDif>20.0) VyskaTonu=20;
  if(VyskaDif<-20.0) VyskaTonu=-20;
  if(VyskaDif>0)
  {
    PocetZv=VyskaTonu*5;
    if(PocetZv>50) PocetZv=50;
    if(VyskaTonu<10) Sound(50-VyskaTonu*2,5);
                else Sound(40-VyskaTonu,5);
  }
  if(VyskaDif<0) Sound(120-VyskaTonu*6,-VyskaTonu*2+10);
}

void Beep3()
{
  signed char DelkaTonu;
  DelkaTonu=(signed char)(VyskaDif*2.0);
  if(VyskaDif>20.0) DelkaTonu=40;
  if(VyskaDif<-20.0) DelkaTonu=-40;
  if(VyskaDif>0) Sound(30,DelkaTonu+5);
  if(VyskaDif<0) Sound(120,-DelkaTonu+5);
}

void Beep4()
{
  signed char VyskaTonu;
  VyskaTonu=(signed char)VyskaDif;
  if(VyskaDif>10.0) VyskaTonu=10;
  if(VyskaDif<-20.0) VyskaTonu=-20;
  if(VyskaDif>0)
  {
    PocetZv=VyskaTonu*5;
    if(VyskaTonu<10) Sound(30,5);
                else Sound(30,5);
  }
  if(VyskaDif<0) Sound(200,-VyskaTonu*2+10);
}

void TestZvuk()
{
  signed char i;
  if(!Zvuk) return;
  lcd_clear();
  lcd_puts("Zvuk +1m");
  i=3;VyskaDif=1.0;
  while(i--)
  {
    switch(Zvuk)
    {
      case 1:Beep1();break;
      case 2:Beep2();break;
      case 3:Beep3();break;
      case 4:Beep4();break;
    }
    Delay01s(10);
  }
  lcd_clear();
  lcd_puts("Zvuk -1m");
  i=3;VyskaDif=-1.0;
  while(i--)
  {
    switch(Zvuk)
    {
      case 1:Beep1();break;
      case 2:Beep2();break;
      case 3:Beep3();break;
      case 4:Beep4();break;
    }
    Delay01s(10);
  }
  i=-9;
  while(i<10)
  {
    VyskaDif=i;
    lcd_clear();
    lcd_puts("Zvuk ");
    if(i<0) { lcd_putch('-');lcd_putch('0'-i); }
       else { lcd_putch('+');lcd_putch('0'+i); }
    lcd_putch('m');
    switch(Zvuk)
    {
      case 1:Beep1();break;
      case 2:Beep2();break;
      case 3:Beep3();break;
      case 4:Beep4();break;
    }
    Delay01s(10);
    i++;
  }
}

void NajdiKonecEEPROM()
{
  TByteInt Zaznam;
  unsigned char i;
  unsigned int AdresaHlavicky;
  AdresaEE=0;AdresaHlavicky=0;
  while(1)
  {
    PosledniEE=AdresaEE;
    SetI2CEEPROMRd(AdresaEE);
    Zaznam.Byte[0]=I2CIn();
    Zaznam.Byte[1]=I2CIn();
    if(Zaznam.Int==0xFFFF)        // Konec zznamu
    {
      ACKDT=1;
      I2CIn();
      I2CStop();
      return;
    }
    if(Zaznam.Int!=0xFFFE)
    {
      AdresaEE+=2;
      goto HledejEA;
    }
    AdresaEE&=0xFFFE;             // Bezpenostn opaten
    Zaznam.Byte[0]=I2CIn();       // Adresa dat
    Zaznam.Byte[1]=I2CIn();
    if(Zaznam.Int==0xFFFF)        // Nen dal adresa (ale jsou data)
    {
      AdresaEE+=10;
      i=8;
      while(i--) I2CIn();         // Peskoit hlaviku
      HledejEA:
      do                          // Najt konec zznamu
      {
        Zaznam.Byte[0]=I2CIn();
        Zaznam.Byte[1]=I2CIn();
        AdresaEE+=2;
        AdresaEE&=0xFFFE;         // Bezpenostn opaten
        if(AdresaEE>=MaxMem-1)    // Pln pam
        {
          AdresaEE=MaxMem-1;
          PosledniEE=MaxMem-1;
          ACKDT=1;
          I2CIn();
          I2CStop();
          return;
        }
      }
      while(Zaznam.Int!=0xFFFF);
      ACKDT=1;
      I2CIn();
      I2CStop();
      SetI2CEEPROMAdr(AdresaHlavicky+2);
      Zaznam.Int=AdresaEE;
      I2COut(Zaznam.Byte[0]);     // Nastavit adresu na konec zznamu
      I2COut(Zaznam.Byte[1]);
      I2CStop();
      DelayMs(15);
      PosledniEE=AdresaEE;
      return;
    }
    AdresaHlavicky=AdresaEE;
    ACKDT=1;
    I2CIn();
    AdresaEE=Zaznam.Int;
  }
}

void VytvorZaznam()
{
  unsigned char i;

  SetI2CEEPROMAdr(AdresaEE);

  I2COut(0xFE);   // FFFE
  I2COut(0xFF);
  I2COut(0xFF);   // FFFF (Adresa nen piazena)
  I2COut(0xFF);
  i=4;
  while(i--) I2COut(Tlak0.Byte[3-i]);
  I2COut(VyskaKopce.Byte[0]);
  I2COut(VyskaKopce.Byte[1]);
  I2COut(Ukladani);
  I2COut(0);
  I2CStop();
  DelayMs(15);
  AdresaEE+=12;
  PosledniEE=AdresaEE;
}

void UlozVysku()
{
  TByteInt vysk;
  if((++CasUkladani)<Ukladani) return;
  CasUkladani=0;
  if(AdresaEE>=MaxMem-1)
  {
    AdresaEE=0;
    return;
  }
  vysk.Int=(unsigned int)(Vyska+10000.0);
  if(vysk.Int>65000) vysk.Int=65000;
  SetI2CEEPROMAdr(AdresaEE);
  I2COut(vysk.Byte[0]);
  I2COut(vysk.Byte[1]);
  I2CStop();
  AdresaEE+=2;
  PosledniEE=AdresaEE;
}

void VymazatPamet()
{
  unsigned char Key;
  lcd_clear();
  lcd_goto(LcdLn1);
  lcd_puts("Smazat?");
  lcd_goto(LcdLn2);
  lcd_putnum(((unsigned long)PosledniEE+2)*100/MaxMem);
  lcd_putch('%');
  do
  {
    Key=ReadKey();
    if(Key==KB_Cancel) return;
  }
  while(Key!=KB_Enter);
  lcd_clear();
  lcd_goto(LcdLn1);
  lcd_puts("Mazu P");
  AdresaEE=0;
  SetI2CEEPROMAdr(0);
  while(AdresaEE<MaxMem)
  {
    I2COut(0xFF);
    if(!(AdresaEE&0xFF))
    {
      lcd_goto(LcdLn2);
      lcd_putnum((unsigned long)AdresaEE*100/MaxMem);
      lcd_putch('%');
    }
    if(!(AdresaEE&0x0F)) 
    {
      I2CStop();
      DelayMs(20);
      SetI2CEEPROMAdr(AdresaEE+1);
    }
    AdresaEE++;
  }
  I2COut(0xFF);
  I2CStop();
  DelayMs(20);
  PosledniEE=0;
}

void PredejEEPROM()
{
  SetI2CEEPROMRd(0);
  while(!TRMT);
  TXREG=cmdData;
  while(!TRMT);  
  TXREG=eeprom_read(EEPROM_EESize);
  while(OutCommand==cmdData)
  {
    while(!TRMT);
    TXREG=I2CIn();
    if(OutPosition>=MaxMem) OutCommand=0;
    OutPosition++;
  }
  ACKDT=1;
  I2CIn();
  I2CStop();
}

void PredejParametry()
{
  TDouble4 TempD4;
  unsigned char i;
  while(!TRMT)
  TXREG=cmdParam;
  while(!TRMT);
  TXREG=10;
  TempD4.Real=Tlak;
  i=4;
  while(i--)
  {
    while(!TRMT);
    TXREG=TempD4.Byte[3-i];
  }
  i=4;
  while(i--)
  {
    while(!TRMT);
    TXREG=Tlak0.Byte[3-i];
  }
  while(!TRMT);
  TXREG=VyskaKopce.Byte[0];
  while(!TRMT);
  TXREG=VyskaKopce.Byte[1];
}

void PrectiSenzor()
{
  TByteSLong Readed;
  I2CStart();
  I2COut(0b11010001);
  ACKDT=0;
  Readed.Byte[2]=I2CIn();
  Readed.Byte[1]=I2CIn();
  ACKDT=1;
  Readed.Byte[0]=I2CIn();
  I2CStop();
  if(Readed.Byte[2]&0x02) Readed.Byte[3]=0xFF;
                     else Readed.Byte[3]=0x00;
  Tlak=Readed.Long;

  Tlak*=Rozsah.Real;
  Tlak+=Posun.Real;
  StaraVyska=Vyska;
  Vyska=7998.0*log(101325.0/Tlak);
  VyskaDif=Vyska-StaraVyska;
  VyskaDif*=2;
  if(VyskaDif>100.0) VyskaDif=100.0;
  if(VyskaDif<-100.0) VyskaDif=-100.0;
  if(Tlak<NejmensiTlak) NejmensiTlak=Tlak;
}

void ReloadEEPROM()
{
  unsigned char i;
  i=4;
  while(i--)
  {
    Posun.Byte[i]=eeprom_read(EEPROM_Posun+i);
    Rozsah.Byte[i]=eeprom_read(EEPROM_Rozsah+i);
    Tlak0.Byte[i]=eeprom_read(EEPROM_Tlak0+i);
  }
  VyskaKopce.Byte[0]=eeprom_read(EEPROM_VyskaKopce);
  VyskaKopce.Byte[1]=eeprom_read(EEPROM_VyskaKopce+1);
  MaxVyska.Byte[0]=eeprom_read(EEPROM_MaxVyska);
  MaxVyska.Byte[1]=eeprom_read(EEPROM_MaxVyska+1);
  Zvuk=eeprom_read(EEPROM_Zvuk);
  MinPokles=eeprom_read(EEPROM_MinPokles);
  MinVzestup=eeprom_read(EEPROM_MinVzestup);
  MaxMem=(0x1000<<eeprom_read(EEPROM_EESize))-1;
  Ukladani=eeprom_read(EEPROM_Ukladani);
}

void ConfigSenzor()
{
  I2CStart();
  I2COut(0b11010000);
  I2COut(0b10011100);
  I2CStop();
}

void KalibrujRozsah()
{
  double TlakStart,TlakPristani,ZmerStartu,ZmerPristani;
  unsigned int VyskaNaPristani;
  unsigned char i;
  lcd_clear();
  lcd_puts("Rozsah m:");
  Long2Text(0);
  if(EditNumber(LcdLn2+2,6))
  {
    // Tlak na startovace, jak by ml bt
    TlakStart=Tlak0.Real*(exp(VyskaKopce.Int/-7998.0));
    // Vypotm zmenou veliinu (z A/D) na startovace
    ZmerStartu=TlakNaStartu-Posun.Real;
    ZmerStartu/=Rozsah.Real;
    // Petu vku msta na pistn
    VyskaNaPristani=Text2Long(6);
    // Zskm tlak, jak by ml bt na pistn
    TlakPristani=Tlak0.Real*(exp(VyskaNaPristani/-7998.0));
    // Vypotm zmenou veliinu (z A/D) na pistn
    ZmerPristani=Tlak-Posun.Real;
    ZmerPristani/=Rozsah.Real;
    // Vpoet rozsahu
    Rozsah.Real=(TlakStart-TlakPristani)/(ZmerStartu-ZmerPristani);
    // Vpoet posunu
    Posun.Real=TlakPristani-ZmerPristani*Rozsah.Real;
    // Uloit do EEPROM
    i=4;
    while(i--)
    {
      eeprom_write(EEPROM_Posun+i,Posun.Byte[i]);
      eeprom_write(EEPROM_Rozsah+i,Rozsah.Byte[i]);
    }
    Tlak=TlakPristani;
  }
}

void ZobrazCas(void)
{
  lcd_clear();
  lcd_goto(LcdLn1);
  lcd_puts("Cas letu");
  lcd_goto(LcdLn2);
  lcd_putuint(CasLetu/120);
  lcd_puts(" m");
}

void main()
{
  unsigned char i,Key;

  ADCON1=0b00001110;  // AN0 m baterku, ostatn jsou digitln
  TRISA=0b11010011;   // -,-,RS,CFG2,RW,E,CFG1,BAT
  TRISB=0b11110000;   // SW1,SW2,SW3,SW4,4xLCD
  TRISC=0b11111011;   // RX,TX,CFG5,SDA,SCL,Piezo,CFG4,CFG3

  lcd_init(FOURBIT_MODE);
  lcd_init(FOURBIT_MODE);
  lcd_init(FOURBIT_MODE);

  SPBRG=3;                // 3686400/(57600bd*16)-1=3
  TXSTA=0b00100110;
  RCSTA=0b10010000;

  //OPTION=0b00001110;    // Watchdog 1:64
  PIE1=0b00100001;        // RCIE,TMR1IE
  PIE2=0;
  INTCON2=0b01110101;     // RBPU
  INTCON3=0;
  INTCON=0b11001000;      // GIE,PEIE,RBIE
  T1CON=0b00000001;

  lcd_clear();
  lcd_goto(LcdLn1);
  lcd_puts("Vario");
  lcd_goto(LcdLn2);
  lcd_puts("MJ 2007");

  ReloadEEPROM();
  I2CSetup();
  ConfigSenzor();

  NejmensiTlak=9999999.0;
  TlakNaStartu=101325.0;
  PocetVarovani=0;
  PosledniEE=0xFFFF;
  AdresaEE=0;

  CasMereni=0;
  Menu=0;
  NeedUpdate=0;

  while(1)
  {
    i=ReadKey();
    if(OutCommand)
    {
      switch(OutCommand)
      {
        case cmdData:PredejEEPROM();break;
        case cmdParam:break;
        default:OutCommand=0;
      }
    }
    if((NeedUpdate)||(i!=Key))
    {
      UpdateLCD=0;
      if(i!=Key)
      {
        Key=i;
        if(Key) Sound(50,10);
        if(Key==KB_Down) if(++Menu>MenuEnd) Menu=0;
        if(Key==KB_Up) if(--Menu>MenuEnd) Menu=MenuEnd;
        if(Key==KB_Right)
        {
          switch(Menu)
          {
            case MenuVyska:
            {
              lcd_clear();
              lcd_puts("Vyska m:");
              Long2Text((unsigned long)VyskaKopce.Int);
              if(EditNumber(LcdLn2+2,6))
              {
                VyskaKopce.Int=Text2Long(6);
                Tlak0.Real=VyskaKopce.Int;
                eeprom_write(EEPROM_VyskaKopce+1,VyskaKopce.Byte[1]);
                eeprom_write(EEPROM_VyskaKopce,VyskaKopce.Byte[0]);
                Tlak0.Real=Tlak/(exp(Tlak0.Real/-7998.0));
                TlakNaStartu=Tlak;
              }
              break;
            }
            case MenuTlak:
            {
              lcd_clear();
              lcd_puts("Tlak pa:");
              Long2Text((unsigned long)(Tlak0.Real));
              if(EditNumber(LcdLn2+2,6))
              {
                Tlak0.Real=Text2Long(6);
                i=4;
                while(i--) eeprom_write(EEPROM_Tlak0+i,Tlak0.Byte[i]);
              }
              break;
            }
            case MenuHmax:
            {
              lcd_clear();
              lcd_puts("H max:");
              Long2Text((unsigned long)MaxVyska.Int);
              if(EditNumber(LcdLn2+2,6))
              {
                MaxVyska.Int=Text2Long(6);
                eeprom_write(EEPROM_MaxVyska+1,MaxVyska.Byte[1]);
                eeprom_write(EEPROM_MaxVyska,MaxVyska.Byte[0]);
              }
              break;
            }
            case MenuNejvyse:
            {
              NejmensiTlak=Tlak;
              break;
            }
            case MenuPamet:
            {
              NajdiKonecEEPROM();
              AdresaEE=0;
              VymazatPamet();
              break;
            }
            case MenuBeep:
            {
              Zvuk++;
              if(Zvuk>4) Zvuk=0;
              eeprom_write(EEPROM_Zvuk,Zvuk);
              break;
            }
            case MenuPokles:
            {
              lcd_clear();
              lcd_puts("Pokl.m:");
              Byte2Text(MinPokles);
              if(EditNumber(LcdLn2+5,3))
              {
                MinPokles=Text2Long(3);
                eeprom_write(EEPROM_MinPokles,MinPokles);
              }
              break;
            }
            case MenuVzestup:
            {
              lcd_clear();
              lcd_puts("Vzes.m:");
              Byte2Text(MinVzestup);
              if(EditNumber(LcdLn2+5,3))
              {
                MinVzestup=Text2Long(3);
                eeprom_write(EEPROM_MinVzestup,MinVzestup);
              }
              break;
            }
            case MenuUkladani:
            {
              lcd_clear();
              lcd_puts("Ukladat:");
              Byte2Text(Ukladani);
              if(EditNumber(LcdLn2+5,3))
              {
                Ukladani=Text2Long(3);
                eeprom_write(EEPROM_Ukladani,Ukladani);
                CasUkladani=0xFE;
                AdresaEE=0;
              }
              break;
            }
            case MenuPosun:
            {
              lcd_clear();
              lcd_puts("Posun m:");
              Long2Text((unsigned long)VyskaKopce.Int);
              if(EditNumber(LcdLn2+2,6))
              {
                // Zskm zmenou veliinu
                Tlak-=Posun.Real;
                // Petu vku msta men
                Posun.Real=Text2Long(6);
                // Zskm tlak, jak by ml bt
                Posun.Real=Tlak0.Real*(exp(Posun.Real/-7998.0));
                // Odetu men
                Posun.Real-=Tlak;
                i=4;
                while(i--) eeprom_write(EEPROM_Posun+i,Posun.Byte[i]);
                Tlak+=Posun.Real;
                Tlak*=Rozsah.Real;
              }
              break;
            }
            case MenuRozsah:
            {
              KalibrujRozsah();
              break;
            }
          }
        }
        if(Key==KB_Left)
        {
          switch(Menu)
          {
            case MenuPamet:
            {
              if(!AdresaEE)
              {
                NajdiKonecEEPROM();
                if(AdresaEE<MaxMem-1) VytvorZaznam();
                                 else AdresaEE=0;
              } else
              {
                AdresaEE=0;
              }
              CasUkladani=0xFE;
              break;
            }
            case MenuBeep:TestZvuk();break;
          }
          while(ReadKey()==KB_Left);
        }
        UpdateLCD=1;
      }
      if(NeedUpdate)
      {
        NeedUpdate=0;
        PrectiSenzor();
        if(((VyskaDif>=MinVzestup)||(-VyskaDif>MinPokles))&&
           ((VyskaDif>=0.2)||(VyskaDif<-0.2)))
        switch(Zvuk)
        {
          case 1:Beep1();break;
          case 2:Beep2();break;
          case 3:Beep3();break;
          case 4:Beep4();break;
        }
        if(Vyska>MaxVyska.Int)
        {
          if(PocetVarovani) VarujVyskou();
        } else PocetVarovani=4;
        UpdateLCD=1;
        i=7;
        while(i--) VarHistory[i+1]=VarHistory[i];
        if(VyskaDif>0) VarHistory[0]=(unsigned char)VyskaDif;
                  else VarHistory[0]=0;
        if(VarHistory[0]>8) VarHistory[0]=8;
        if(AdresaEE) UlozVysku();
        if(OutCommand==cmdParam) PredejParametry();
      }
      if(UpdateLCD)
      {
        lcd_clear();
        lcd_goto(LcdLn1);
        switch(Menu)
        {
          case MenuHV:ZobrazHV();break;             // Relativn vka a vario
          case MenuVario:ZobrazVario();break;       // Vario a graf
          case MenuVyska:ZobrazVysku();break;       // Absolutn vka podle standardnho tlaku
          case MenuTlak:ZobrazTlak();break;         // Tlak, Edit:Tlak pepoten na hladinu moe
          case MenuHmax:ZobrazHmax();break;         // Maximln letov hladina - Edit:Alarm
          case MenuNejvyse:ZobrazNejvyse();break;   // Maximln dosaen relativn vka nad kopcem, Edit:Vka kopce
          case MenuBeep:SelectBeep();break;         // Vbr zvuku
          case MenuPokles:PoklesBeep();break;       // Minimln pokles pi kterm se zane ppat
          case MenuVzestup:VzestupBeep();break;     // Minimln vzestup pi kterm se zane ppat
          case MenuBaterka:ZmerBaterku();break;     // Napt baterky
          case MenuUkladani:NastUkladani();break;   // Ukldn dat, kad n-t vzorek
          case MenuPamet:ZobrazPamet();break;       // Velikost obsazen pamti, Edit:Smazat pam/spustit zznam
          case MenuPosun:ZobrazPosun();break;       // Kalibrace posunu
          case MenuRozsah:ZobrazRozsah();break;     // Kalibrace rozsahu a posunu
          case MenuCas:ZobrazCas();break;           // Zobraz celkov as ve vzduchu
        }
      }
    }
  }
}
